/* max132.c

 * Copyright (C) 2002 Claudio Girardi
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA
 */


#include <stdio.h>
#include <unistd.h> /* for usleep() and getuid() */
#include <asm/io.h>


#define ADC_POWER 0x30		/* power from P0 and SLCT_OUT */
#define ADC_SCLK  0x04		/* serial clock from D2 */
#define ADC_CS    0x10		/* chip select from D4 */
#define ADC_EOC   0x40		/* end-of-conversion to ACK */
#define ADC_DOUTMASK 0x80	/* serial data out thru BUSY */
#define ADC_DINMASK  0x08	/* serial data in thru D3 */
#define ADC_DELAY 10             /* delay between parallel port commands */

#define BIT0  (1 << 0)
#define BIT1  (1 << 1)
#define BIT2  (1 << 2)
#define BIT3  (1 << 3)
#define BIT4  (1 << 4)
#define BIT5  (1 << 5)
#define BIT6  (1 << 6)
#define BIT7  (1 << 7)

static int port_base;		/* printer port base address */


int init_hw(int base)
{
  uid_t real_uid;
  gid_t real_gid;

  /* checking I/O permissions */
  if (ioperm(base, 3, 1)) {
    /* can not access the parallel port */
    return -1;
  } else {
    port_base = base;
    /* OK, now dropping root privileges */
    real_uid = getuid();
    setuid(real_uid);
    real_gid = getgid();
    setgid(real_gid);
    return 0;
  }
}


void close_hw(void)
{
  /* nothing to do */
}


static void rwbyte(unsigned short outbyte, unsigned short *inbyte)
{
  unsigned short rbyte = 0, rbit, outbit, i;
  long int j;

  /* power up the ADC */
  outb(ADC_POWER, port_base + 1);
  /* wait for EOC */
  /* FIXME: should not wait if  just selecting channel ? */
  /* i.e. if ((outbyte & 0x01) == 0x00) */
  while ((inb(port_base + 1) & ADC_EOC) != 0);

  /* before clocking in data put CS and SCLK low */
  outb(0, port_base);

  for (j = 0; j < ADC_DELAY; j++);
  /* read and write 8 bits */
  rbyte = 0;
  for (i = 1; i <= 8; i++) {

    /* get MSB (D7) */
    outbit = (outbyte & 128) / 128;
    /* output bit and set SCLK */
    for (j = 0; j < ADC_DELAY; j++);
    outb((unsigned char) ((outbit * ADC_DINMASK)), port_base);
    for (j = 0; j < ADC_DELAY; j++);
    outb((unsigned char) (ADC_SCLK | (outbit * ADC_DINMASK)), port_base);

    /* read bit */
    /* this is a bit unclear from the data sheet, but seems that the */
    /* output data should be read before clearing SCLK */
    for (j = 0; j < ADC_DELAY; j++);
    rbit = ((inb(port_base + 1) & ADC_DOUTMASK) / ADC_DOUTMASK);
    rbyte = (rbyte << 1) + rbit;

    /* clear SCLK */
    for (j = 0; j < ADC_DELAY; j++);
    outb((unsigned char) ((outbit * ADC_DINMASK)), port_base);
    outbyte = outbyte << 1;
  }

  /* set CS */
  for (j = 0; j < ADC_DELAY; j++);
  outb((unsigned char) (ADC_CS), port_base);
  for (j = 0; j < ADC_DELAY; j++);

  (*inbyte) = rbyte;
}


float get_sample(int channel)
{
  unsigned short outbyte, inbyte;
  long int result;
  float value;

  result = 0;

  if (channel == -1) { /* channel -1 means "read the ADC offset" */
    /* read zero offset */
    channel = 0;
    outbyte = 0xd2;
  } else {
    /* read previously selected channel */
    outbyte = 0xc2;
  }

  /* select input channel using the user programmable output bits */
  /* (see schematic and data sheet ) */
  rwbyte(((channel & 0x0f) << 4) + 0x01, &inbyte);
  usleep(250000); /* wait for 0.25 sec, allowing the input to stabilize */


  /* start conversion, read Output Register 0, next read Output Status Reg. */
  rwbyte(outbyte, &inbyte);
  /* inbyte contains now invalid data */

  /* continue conversion, read Output Status Register, next read Out. Reg. 1 */
  outbyte = 0x44;
  rwbyte(outbyte, &inbyte);

#ifdef DEBUG
  printf("Collision  : %i\n", (inbyte & 128) / 128);
  printf("EOC        : %i\n", (inbyte & 64) / 64);
  printf("Integrating: %i\n", (inbyte & 32) / 32);
  printf("Sleeping   : %i\n", (inbyte & 16) / 16);
  printf("Polarity - : %i\n", (inbyte & 8) / 8);
#endif

  /* extract the B2-B0 bits from the Output Status Register */
  result = (inbyte & 0x07);

  /* continue conversion, read Output Register 1, next read Out. Reg. 0 */
  outbyte = 0x40;
  rwbyte(outbyte, &inbyte);

  /* add higher order bits B18-B11 */
  result += (inbyte * 2048);

  /* dummy write, for reading Output Register 0 */
  outbyte = ((channel & 0x0f) * 16) + 0x01;
  rwbyte(outbyte, &inbyte);

  /* add lower order bits B10-B3 */
  result += (inbyte * 8);

  /* adjust the sign of the result */
  if ((result & 0x40000) == 0x40000) {
    /* clear the sign bit */
    result = result - 524289;
  }
#ifdef DEBUG
  printf("Result = %li\n", result);
#endif

  /*  input voltage; the minus sign is due to hardware connections */
  value = -0.512 * result / 262144.0;

  return value;
}
