/* test_max132.c

 * Copyright (C) 2002 Claudio Girardi
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "max132.h"

#define VERSION "0.1alpha"


static char *program_name;
static long num_acq = 100;	/* number of samples per channel to read */
static int port_base = 0x378;	/* printer port base address */


static void usage(void)
{
  fprintf(stderr,
	  "%s " VERSION ", by Claudio Girardi - IN3OTD\n"
	  "Usage: %s [options]\n"
  "options: -n : number of samples per channel to acquire (default %li)\n"
	  "         -p : parallel port address (default 0x%x)\n"
	  "         -h : print this help\n"
	  "         -v : print version information\n",
	  program_name, program_name, num_acq, port_base);
  fprintf(stderr, "\n");
  fprintf(stderr, "Mail bug reports and suggestions to <in3otd@qsl.net>\n");
  fprintf(stderr, "\n");
  exit(1);
}


static void version(void)
{
  fprintf(stderr,
	  /* State the canonical name for the program */
	  "test_max132 " VERSION "\n"
	  "Copyright (C) 2002 Claudio Girardi <in3otd@qsl.net>\n"
	  "This program is distributed in the hope that it will be useful,\n"
	  "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
	  "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
	  "GNU General Public License for more details.\n"
	  "\n");
}


static void parse_command_line(int argc, char *argv[])
{
  int optc;			/* For getopt */

  /* Parse the options */
  while ((optc = getopt(argc, argv, "n:hv")) != -1) {
    switch (optc) {
    case 'n':
      num_acq = atol(optarg);
      break;
    case 'p':
      port_base = atoi(optarg);
      break;
    case 'h':
      usage();
      exit(0);
      break;
    case 'v':
      version();
      exit(0);
      break;
    default:
      usage();
      exit(0);
    }
  }
}


float getclock(void)
{

  clock_t nclock;
  float nsec;

  nclock = clock();
  nsec = (float) nclock / CLOCKS_PER_SEC;

  return nsec;

}


int main(int argc, char *argv[])
{

  int i, j;
  float adc_zero, value;
  double mean[4], sqrmean[4];
  float start, stop;


  /* Construct the name of the executable, without the directory part.  */
  program_name = strrchr(argv[0], '/');
  if (!program_name)
    program_name = argv[0];
  else
    ++program_name;

  parse_command_line(argc, argv);

  /* initialize ADC */
  if (init_hw(port_base) == -1) {
    fprintf(stderr, "%s: Couldn't open the parallel port at address 0x%x; ", program_name, port_base);
    fprintf(stderr, "you need root privileges to run this program.\n");
    exit(1);
  }

  /* acquire ADC zero */
  fprintf(stderr, "Acquiring ADC zero... ");
  adc_zero = 0.0;
  for (i=0; i<10; i++) {
    adc_zero += get_sample(-1);
  }
  adc_zero /= 10;
  fprintf(stderr, "done\n");

  /* initialize variables */
  sqrmean[0] = sqrmean[1] = sqrmean[2] = sqrmean[3] = 0.0;
  mean[0] = mean[1] = mean[2] = mean[3] = 0.0;


  fprintf(stderr, "Acquiring %li samples... ", num_acq);
  /* get current time */
  start = getclock();

  /* read num_acq samples per channel */
  for (i = 0; i < num_acq; i++) {
    /* read all the four channel in sequence */
    for (j = 0; j < 4; j++) {
      value = get_sample(j);
      /* accumulate the mean square value */
      sqrmean[j] += (value * value / num_acq);
      /* accumulate the mean value */
      mean[j] += (value / num_acq);
      /* output the values in a row */
      printf("%0.7f\t", value);
    }
    /* go to next line for a new reading cycle */
    printf("\n");
  }

  /* get current time */
  stop = getclock();

  /* done with acquisition */
  fprintf(stderr, "done\n");

  fprintf(stderr, "\n");
  /* display mean acquisition time */
  fprintf(stderr, "Mean acquisition time for a set of four channels : %e s\n", (stop - start) / num_acq);

  fprintf(stderr, "\n");
  /* display channels standard deviation */
  for (j = 0; j < 4; j++) {
    fprintf(stderr, "Standard deviation for channel %i = %e V\n", j, sqrt(sqrmean[j] - mean[j] * mean[j]));
  }

  fprintf(stderr, "\n");
  /* display channels mean values */
  for (j = 0; j < 4; j++) {
    fprintf(stderr, "Mean value for channel %i = %e V\n", j, mean[j]);
  }
  fprintf(stderr, "\n");

  exit(0);
}

